/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Server EAL package.
 *
 * The Initial Developer of the Original Code is Nokia Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contact: Andrey Petrov <andrey.petrov@nokia.com>
 *
 * ***** END LICENSE BLOCK ***** */


#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include <dbus/dbus-glib.h>

#define VERSION myversion

#include <gtk/gtk.h>
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* 1. Include Web Object header */
#include <gweb.h>
#include <neteal.h>

//#undef USE_HILDON
//#undef USE_GTKADI
#define USE_WINVIEW 1
#define MAX_WIN_COUNT 1
static int win_count = 0;

#define EAL_TEST_APP_TEST_PAGE    "about:"

#ifdef USE_HILDON
#include<hildon-widgets/hildon-window.h>
#include<hildon-widgets/hildon-program.h>
#endif

#ifdef USE_GTKADI
#include <gtkadi.h>
#define WIDGET_MANAGER GtkAdi
#ifdef USE_WINVIEW
#define DEFAULT_VIEW GTK_ADI_VIEW_WIN
#else
#define DEFAULT_VIEW GTK_ADI_VIEW_TAB
#endif
#else
#define WIDGET_MANAGER void
#endif

/* Window destroy callback */
static void window_destroy( GtkWidget *widget, gpointer data )
{
    printf("Destroy window: %p\n", widget);
    gtk_main_quit ();
}

void on_focus_child(GObject * adiview, GtkWidget * child, void *data)
{
    printf("Focus child: '%s',%p\n", gtk_widget_get_name(child), child);
}

gboolean  plug_removed       (GtkSocket *socket_,
                             gpointer   user_data) {
	g_critical ("plug disconnected - server crashed.");
	return FALSE;
}			     

typedef void (* SetEnvFunc) (void);
typedef GWeb* (* GetWebNewFunc) (void);

GWeb *web; /* Global Web interface object */
GtkWidget * wm = NULL;
guint32 connection = 0;
static void create_container_for_widget (WIDGET_MANAGER * manager,
                                         GtkWidget ** cont,
                                         GtkWidget * widget,
                                         void *data)
{
    printf("Create_container_for_widget: %p", widget);
    GtkWidget *window = NULL;
#ifdef USE_HILDON
    window = hildon_window_new ();
    HildonProgram * pr = hildon_program_new();
    hildon_program_add_window (pr, HILDON_WINDOW (window));
#else
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
#endif
    g_signal_connect (G_OBJECT (window), "destroy",
                      G_CALLBACK (window_destroy), NULL);
    *cont = window;
    gtk_widget_show (window);
    printf(" Cont: %p\n", *cont);
    return;
}

gboolean ask_for_shutdown(gpointer tmp) {
    

    neteal_connection_terminate(neteal_get_current_connection());
    return FALSE;
}

void myerrorhandler (guint32 connectionid, gboolean is_expected, gpointer user_data) 
{
    g_debug("connection id =%x lost, is_expected %x, ud=%x", connectionid, is_expected, user_data);
}


gboolean open_new_engine(gchar * url)
{

    guint32 xid = 0;
    GtkSocket *socket = NULL;
    if (++win_count > MAX_WIN_COUNT)
        return FALSE;

    printf("Open new Engine: %s\n", url);
    GWebEngine *engine; /* Web engine */
    GtkWidget *engine_widget; /* Web engine widget - for faster access (and shorter code).*/


    engine = g_web_new_web_engine_window_with_context(web, NULL, FALSE, FALSE);
    g_debug("client: got engine %x", (guint) engine);


    GtkWindow *main_window;

    main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (main_window),
			"test of embedded mozilla socket");



/*
#ifdef USE_GTKADI
    gtk_adi_add (GTK_ADI (wm), engine_widget);
    gtk_adi_set_child_tab (GTK_ADI (wm), engine_widget, TRUE);
#else
#ifdef USE_WINVIEW
    GtkWidget *tmp_win = NULL;
    create_container_for_widget (NULL, &tmp_win, engine_widget, NULL);
    gtk_container_add(GTK_CONTAINER(tmp_win), engine_widget);
#else
    gtk_notebook_append_page(wm, engine_widget, NULL);
    gtk_notebook_set_tab_label_text(wm, engine_widget, url);
    gtk_notebook_next_page(wm);
#endif
#endif

    
*/

      GtkWidget *ss = GTK_WIDGET(gtk_socket_new());  
      
      gtk_container_add (GTK_CONTAINER (main_window), GTK_WIDGET(ss));
      gtk_widget_realize (ss);

      gtk_widget_show (ss);
      gtk_widget_show (main_window);

      guint32 hwidget =
        g_web_engine_embed_engine_widget_xid ( 
                    engine, 
                    TRUE, 
                    GDK_WINDOW_XWINDOW(gdk_window_get_toplevel( ss->window ) ), 
                    &xid);

        gtk_socket_add_id ((GtkSocket *) ss, xid);
        gtk_widget_show (ss);


    g_debug("eal-client: done, got handle=%x", hwidget);
    /* 8. Connect some engine widget signals (Ex.: Set 'URL changed' and 'start loading' callbacks.) */

    GObject *en_notify; /* Global Web implementation object */
    
    g_debug("ealclient : getting engine notifier");

    g_signal_connect(ss, "plug-removed",
                     G_CALLBACK(plug_removed), NULL);



    en_notify = g_web_engine_get_engine_notifier(engine);
    g_debug("ealcient: got %x engine notifier", (guint) en_notify);
    
    g_signal_connect_swapped(en_notify, G_WEBWIDGET_SIGNAL_URL_CHANGED,
                             G_CALLBACK(g_web_widget_signal_url_changed), NULL);
    g_signal_connect(en_notify, G_WEBWIDGET_SIGNAL_TITLE,
                     G_CALLBACK(g_web_widget_signal_title), NULL);

    g_signal_connect(en_notify, G_WEBWIDGET_SIGNAL_START_LOADING,
                     G_CALLBACK(g_web_widget_signal_start_loading), NULL);
    g_signal_connect(en_notify, G_WEBWIDGET_SIGNAL_FINISHED_LOADING,
                     G_CALLBACK(g_web_widget_signal_finished_loading), NULL);


    g_debug("ealclient: load_url");

    g_web_engine_load_url(engine, url);
    g_debug("ealclient: load_url done");

    return TRUE;
}

void myerrorhandler2 (guint32 connectionid, gboolean is_expected, gpointer user_data) {

    g_debug("connection %d died, is_expected %d", connectionid, is_expected);
    

}


int main( int argc, char* argv[] )
{
    GObject *global; /* Global Web implementation object */
    gchar * url = NULL;
    GArray *g;
    guint32 server;
    
    /* Init the gtk */
    gtk_init( &argc, &argv );

     g_thread_init(NULL);
     dbus_g_thread_init ();


    printf("\n\nStart Test Browser Eal Application\n");

    if (argc == 0)
        url = NULL;

    if (argc > 1)
        url = g_strdup_printf("%s",argv[1]);
	
    neteal_init();
    	
    g = neteal_servers_list();
    
    if (!g->len) {
        g_debug("no servers around, creating our own");
	server = neteal_server_create_fork("mytestserver");
    } else {
       server = g_array_index(g, guint32, 0);
       g_debug("there are %d servers, using first [%d]", g->len, server);
    }
    
    g_array_free(g, TRUE);
    g_debug("using server %d", server);
    
    connection = neteal_server_connect(server);
    
    neteal_set_connection_errorhandler(connection, myerrorhandler, 42);
    
    neteal_set_current(connection);
    
    g_debug("got connection %x", connection);
    
    g = neteal_list_webs();
    g_debug("there are %d web[s] created on the server", g->len);
    
    if (!g->len) {
        g_debug("therefore, creating our own");
        web = (GWeb*) g_web_new_neteal(argv[2]);
        g_debug("current engine is %s", g_web_getname(web));
    } else {
       web = (GWeb*) g_array_index(g, guint32, 0);
        g_debug("current engine is %s", g_web_getname(web));       
    }
    
    g_debug("using web %x", (guint)web);

//    g_timeout_add(2000, ask_for_shutdown, NULL);

    /* 3. Perform some initializations (Ex.: Set User Agent string.) */

//    g_web_set_ua_string(web, "browser-ui-test ", VERSION);

    /* 4. Get Global Web implementation object */

   g_debug("eal-client : getting global obj");

   global = g_web_get_global(web);

   g_signal_connect(global, G_WEB_SIGNAL_MODAL_DIALOG, G_CALLBACK(g_web_signal_modal_dialog), NULL);

    /* Initialize widget manager stuff */
#ifdef USE_GTKADI
    wm = gtk_adi_new ();
    gtk_adi_change_view (GTK_ADI (wm),  DEFAULT_VIEW);
    g_object_ref(wm);
    g_signal_connect(G_OBJECT(wm), ADI_GET_CONT_S,
                     G_CALLBACK (create_container_for_widget), NULL);
    g_signal_connect(G_OBJECT(wm), ADI_FOCUS_CHILD_S,
                     G_CALLBACK (on_focus_child), NULL);
    g_signal_connect(G_OBJECT(wm), ADI_CLOSE_CHILD_S,
                     G_CALLBACK (window_destroy), NULL);

#else
#ifndef USE_WINVIEW
    wm = gtk_notebook_new();
    gtk_widget_show(wm);
    GtkWidget *tmp_win = NULL;
    create_container_for_widget(NULL, &tmp_win,  wm, NULL);
    gtk_container_add(tmp_win, wm);
#endif
#endif

    /* Begin the main app */
    open_new_engine(url);
//    g_timeout_add(3000, (GSourceFunc) open_new_engine, "http://www.randomgoogle.com/?Random");
//    g_timeout_add(3000, (GSourceFunc) open_new_engine, "javascript:alert('hi');");

    gtk_main();
    return 0;
}

GWebDialogButton g_web_signal_modal_dialog(GObject *global,
                                           const gchar* caption,
                                           const gchar* message,
                                           GWebDialogMessageType type,
                                           GWebDialogButton button1,
                                           GWebDialogButton button2,
                                           GWebDialogButton button3)
{
    return 0;
}

void g_web_widget_signal_title(GObject *engine_widget,
                               const gchar *title,
                               GObject *browser_window)
{
    printf("!!!!!!!!!!!!!!!!!!!!!!!!Title changed signal, new title is: %s\n", title);
}

void g_web_widget_signal_url_changed(GObject *browser_window,
                                     const gchar *url)
{
    printf("!!!!!!!!!!!!!!!!!!!!!!!!Url changed signal, new url is: %s\n", url);
}

void g_web_widget_signal_start_loading(GObject *engine_widget,
                                       const gchar *url,
                                       GObject *browser_window)
{
    printf("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!Start Loading signal, with url: %s\n", url);
}

void g_web_widget_signal_finished_loading(GObject *engine_widget,
                                          gint finish_status,
                                          gint page_size,
                                          gboolean has_frames,
                                          GObject *browser_window)
{
    printf("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!Finished Loading signal\n");
}
