/* vim:set ts=2 sw=4 sts=2 et cindent: */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the EAL Test package.
 *
 * The Initial Developer of the Original Code is Nokia Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contact: Leonid Zolotarev <leonid.zolotarev@nokia.com>
 *
 * ***** END LICENSE BLOCK ***** */

#include <gtk/gtk.h>
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* 1. Include Web Object header */
#include <gweb.h>

//#undef USE_HILDON
//#undef USE_GTKADI
#define USE_WINVIEW 1
#define MAX_WIN_COUNT 1
static win_count = 0;

#define EAL_TEST_APP_TEST_PAGE    "about:"

#ifdef USE_HILDON
#include<hildon-widgets/hildon-window.h>
#include<hildon-widgets/hildon-program.h>
#endif

#ifdef USE_GTKADI
#include <gtkadi.h>
#define WIDGET_MANAGER GtkAdi
#ifdef USE_WINVIEW
#define DEFAULT_VIEW GTK_ADI_VIEW_WIN
#else
#define DEFAULT_VIEW GTK_ADI_VIEW_TAB
#endif
#else
#define WIDGET_MANAGER void
#endif

/* Window destroy callback */
static void window_destroy( GtkWidget *widget, gpointer data )
{
    printf("Destroy window: %p\n", widget);
    gtk_main_quit ();
}

void on_focus_child(GObject * adiview, GtkWidget * child, void *data)
{
    printf("Focus child: '%s',%p\n", gtk_widget_get_name(child), child);
}

typedef void (* SetEnvFunc) (void);
typedef GWeb* (* GetWebNewFunc) (void);

GWeb *web; /* Global Web interface object */
GtkWidget * wm = NULL;

static void create_container_for_widget (WIDGET_MANAGER * manager,
                                         GtkWidget ** cont,
                                         GtkWidget * widget,
                                         void *data)
{
    printf("Create_container_for_widget: %p", widget);
    GtkWidget *window = NULL;
#ifdef USE_HILDON
    window = hildon_window_new ();
    HildonProgram * pr = hildon_program_new();
    hildon_program_add_window (pr, HILDON_WINDOW (window));
#else
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
#endif
    g_signal_connect (G_OBJECT (window), "destroy",
                      G_CALLBACK (window_destroy), NULL);
    *cont = window;
    gtk_widget_show (window);
    printf(" Cont: %p\n", *cont);
    return;
}



gboolean open_new_engine(gchar * url)
{
    if (++win_count > MAX_WIN_COUNT)
        return FALSE;
    printf("Open new Engine: %s\n", url);
    GWebEngine *engine; /* Web engine */
    GObject *engine_widget; /* Web engine widget - for faster access (and shorter code).*/
    engine = g_web_new_web_engine_window_with_context(web, NULL, FALSE, FALSE);
    /* 7. Get Web Engine widget */
    engine_widget = g_web_engine_get_engine_widget(engine);
    if (!engine_widget)
    {
        printf("Error: Can't init engine widget, exit..\n");
        return 1;
    }
    gtk_widget_show(engine_widget);
#ifdef USE_GTKADI
    gtk_adi_add (GTK_ADI (wm), engine_widget);
    gtk_adi_set_child_tab (GTK_ADI (wm), engine_widget, TRUE);
#else
#ifdef USE_WINVIEW
    GtkWidget *tmp_win = NULL;
    create_container_for_widget (NULL, &tmp_win, engine_widget, NULL);
    gtk_container_add(tmp_win, engine_widget);
#else
    gtk_notebook_append_page(wm, engine_widget, NULL);
    gtk_notebook_set_tab_label_text(wm, engine_widget, url);
    gtk_notebook_next_page(wm);
#endif
#endif
    /* 8. Connect some engine widget signals (Ex.: Set 'URL changed' and 'start loading' callbacks.) */
    GObject *en_notify; /* Global Web implementation object */
    en_notify = g_web_engine_get_engine_notifier(engine);
    g_signal_connect_swapped(en_notify, G_WEBWIDGET_SIGNAL_URL_CHANGED,
                             G_CALLBACK(g_web_widget_signal_url_changed), NULL);
    g_signal_connect(en_notify, G_WEBWIDGET_SIGNAL_TITLE,
                     G_CALLBACK(g_web_widget_signal_title), NULL);
    g_signal_connect(en_notify, G_WEBWIDGET_SIGNAL_START_LOADING,
                     G_CALLBACK(g_web_widget_signal_start_loading), NULL);
    g_signal_connect(en_notify, G_WEBWIDGET_SIGNAL_FINISHED_LOADING,
                     G_CALLBACK(g_web_widget_signal_finished_loading), NULL);

    g_web_engine_load_url(engine, url);

    return TRUE;
}

static GWeb*
init_engine_by_name(const gchar * engine)
{
    printf("Init_engine_by_name: '%s'\n", engine);

    gchar *engine_name = NULL;
    SetEnvFunc  web_set_env;
    GetWebNewFunc  web_get_new;

    if (engine)
        engine_name = g_strdup_printf("/usr/lib/lib%s-eal.so.0", engine);
    else
        engine_name = g_strdup("/usr/lib/libopera-eal.so.0");

    printf("\tOpening engine: %s\n", engine_name);
    GModule* module = g_module_open(engine_name, G_MODULE_BIND_LAZY);
    g_free(engine_name);
    if (g_module_error ())
        printf("\tModule error: %s\n", g_module_error ());
    if (!g_module_symbol (module, "g_web_set_env", (gpointer *)&web_set_env))
        printf("\tg_web_set_env Function opened unsuccessefully\n");
    else
        printf("\tg_web_set_env Function opened successefully\n");

    if (!g_module_symbol (module, "g_web_new", (gpointer *)&web_get_new))
        printf("\tg_web_new Function opened unsuccessefully\n");
    else
        printf("\tg_web_new Function opened successefully\n");

    web_set_env();
    /* 2. Instantiate Global Web interface object */
    return web_get_new ();
}

int main( int argc, char* argv[] )
{
    GObject *global; /* Global Web implementation object */
    gchar * url = NULL;

    /* Init the gtk */
    
    g_thread_init(NULL);
    
    gtk_init( &argc, &argv );

    printf("\n\nStart Test Browser Eal Application\n");

    if (argc == 0)
        url = NULL;

    if (argc > 1)
        url = g_strdup_printf("%s",argv[1]);

    web = init_engine_by_name(argv[2]);

    /* 3. Perform some initializations (Ex.: Set User Agent string.) */
    /* 4. Get Global Web implementation object */
    global = g_web_get_global(web);
    /* 5. Connect some global signals (Ex.: Set modal dialog callback.) */
    g_signal_connect(global, G_WEB_SIGNAL_MODAL_DIALOG, G_CALLBACK(g_web_signal_modal_dialog), NULL);

    /* Initialize widget manager stuff */
#ifdef USE_GTKADI
    wm = gtk_adi_new ();
    gtk_adi_change_view (GTK_ADI (wm),  DEFAULT_VIEW);
    g_object_ref(wm);
    g_signal_connect(G_OBJECT(wm), ADI_GET_CONT_S,
                     G_CALLBACK (create_container_for_widget), NULL);
    g_signal_connect(G_OBJECT(wm), ADI_FOCUS_CHILD_S,
                     G_CALLBACK (on_focus_child), NULL);
    g_signal_connect(G_OBJECT(wm), ADI_CLOSE_CHILD_S,
                     G_CALLBACK (window_destroy), NULL);

#else
#ifndef USE_WINVIEW
    wm = gtk_notebook_new();
    gtk_widget_show(wm);
    GtkWidget *tmp_win = NULL;
    create_container_for_widget(NULL, &tmp_win,  wm, NULL);
    gtk_container_add(tmp_win, wm);
#endif
#endif

    /* Begin the main app */
    open_new_engine(url);
    g_timeout_add(3000, open_new_engine, "http://www.randomgoogle.com/?Random");

    gtk_main();
    return 0;
}

GWebDialogButton g_web_signal_modal_dialog(GObject *global,
                                           const gchar* caption,
                                           const gchar* message,
                                           GWebDialogMessageType type,
                                           GWebDialogButton button1,
                                           GWebDialogButton button2,
                                           GWebDialogButton button3)
{
    return 0;
}

void g_web_widget_signal_title(GObject *engine_widget,
                               const gchar *title,
                               GObject *browser_window)
{
    printf("Title changed signal, new title is: %s\n", title);
}

void g_web_widget_signal_url_changed(GObject *browser_window,
                                     const gchar *url)
{
    printf("Url changed signal, new url is: %s\n", url);
}

void g_web_widget_signal_start_loading(GObject *engine_widget,
                                       const gchar *url,
                                       GObject *browser_window)
{
    printf("Start Loading signal, with url: %s\n", url);
}

void g_web_widget_signal_finished_loading(GObject *engine_widget,
                                          gint finish_status,
                                          gint page_size,
                                          gboolean has_frames,
                                          GObject *browser_window)
{
    printf("Finished Loading signal\n");
}
